@@ -1,18 +1,41 @@
 #pragma once
 
-#if SIMDJSON_EXCEPTIONS
-
-//
-// Interface
-//
+#include "json_benchmark/string_runner.h"
+#include "json_benchmark/point.h"
+#include <vector>
+#include <random>
 
 namespace kostya {
-template<typename T> static void Kostya(benchmark::State &state);
-namespace sum {
-template<typename T> static void KostyaSum(benchmark::State &state);
-}
 
-using namespace simdjson;
+using namespace json_benchmark;
+
+static const simdjson::padded_string &get_built_json_array();
+
+template<typename I>
+struct runner : public string_runner<I> {
+  std::vector<point> result;
+
+  runner() : string_runner<I>(get_built_json_array()) {}
+
+  bool before_run(benchmark::State &state) {
+    if (!string_runner<I>::before_run(state)) { return false; }
+    result.clear();
+    return true;
+  }
+
+  bool run(benchmark::State &) {
+    return this->implementation.run(this->json, result);
+  }
+
+  template<typename R>
+  bool diff(benchmark::State &state, runner<R> &reference) {
+    return diff_results(state, result, reference.result, I::DiffFlags);
+  }
+
+  size_t items_per_iteration() {
+    return result.size();
+  }
+};
 
 static void append_coordinate(std::default_random_engine &e, std::uniform_real_distribution<> &dis, std::stringstream &myss) {
   using std::endl;
@@ -49,47 +72,15 @@ static std::string build_json_array(size_t N) {
   return answer;
 }
 
-static const padded_string &get_built_json_array() {
-  static padded_string json = build_json_array(524288);
+static const simdjson::padded_string &get_built_json_array() {
+  static simdjson::padded_string json = build_json_array(524288);
   return json;
 }
 
-struct my_point {
-  double x;
-  double y;
-  double z;
-  simdjson_really_inline bool operator==(const my_point &other) const {
-    return x == other.x && y == other.y && z == other.z;
-  }
-  simdjson_really_inline bool operator!=(const my_point &other) const { return !(*this == other); }
-};
+struct simdjson_dom;
 
-simdjson_unused static std::ostream &operator<<(std::ostream &o, const my_point &p) {
-  return o << p.x << "," << p.y << "," << p.z << std::endl;
+template<typename I> simdjson_really_inline static void kostya(benchmark::State &state) {
+  run_json_benchmark<runner<I>, runner<simdjson_dom>>(state);
 }
 
 } // namespace kostya
-
-//
-// Implementation
-//
-#include <vector>
-#include "event_counter.h"
-#include "dom.h"
-#include "json_benchmark.h"
-
-namespace kostya {
-
-template<typename T> static void Kostya(benchmark::State &state) {
-  JsonBenchmark<T, Dom>(state, get_built_json_array());
-}
-
-namespace sum {
-template<typename T> static void KostyaSum(benchmark::State &state) {
-  JsonBenchmark<T, Dom>(state, get_built_json_array());
-}
-}
-
-} // namespace kostya
-
-#endif // SIMDJSON_EXCEPTIONS
