@@ -121,28 +121,32 @@ simdjson_really_inline document_stream::~document_stream() noexcept {
 #endif
 }
 
+simdjson_really_inline document_stream::iterator::iterator() noexcept
+  : stream{nullptr}, finished{true} {
+}
+
 simdjson_really_inline document_stream::iterator document_stream::begin() noexcept {
   start();
   // If there are no documents, we're finished.
-  return iterator(*this, error == EMPTY);
+  return iterator(this, error == EMPTY);
 }
 
 simdjson_really_inline document_stream::iterator document_stream::end() noexcept {
-  return iterator(*this, true);
+  return iterator(this, true);
 }
 
-simdjson_really_inline document_stream::iterator::iterator(document_stream& _stream, bool is_end) noexcept
+simdjson_really_inline document_stream::iterator::iterator(document_stream* _stream, bool is_end) noexcept
   : stream{_stream}, finished{is_end} {
 }
 
-simdjson_really_inline simdjson_result<element> document_stream::iterator::operator*() noexcept {
+simdjson_really_inline document_stream::iterator::reference document_stream::iterator::operator*() noexcept {
   // Note that in case of error, we do not yet mark
   // the iterator as "finished": this detection is done
   // in the operator++ function since it is possible
   // to call operator++ repeatedly while omitting
   // calls to operator*.
-  if (stream.error) { return stream.error; }
-  return stream.parser->doc.root();
+  if (stream->error) { return stream->error; }
+  return stream->parser->doc.root();
 }
 
 simdjson_really_inline document_stream::iterator& document_stream::iterator::operator++() noexcept {
@@ -156,16 +160,16 @@ simdjson_really_inline document_stream::iterator& document_stream::iterator::ope
   //
   // Note that setting finished = true is essential otherwise
   // we would enter an infinite loop.
-  if (stream.error) { finished = true; }
-  // Note that stream.error() is guarded against error conditions
-  // (it will immediately return if stream.error casts to false).
-  // In effect, this next function does nothing when (stream.error)
+  if (stream->error) { finished = true; }
+  // Note that stream->error() is guarded against error conditions
+  // (it will immediately return if stream->error casts to false).
+  // In effect, this next function does nothing when (stream->error)
   // is true (hence the risk of an infinite loop).
-  stream.next();
+  stream->next();
   // If that was the last document, we're finished.
   // It is the only type of error we do not want to appear
   // in operator*.
-  if (stream.error == EMPTY) { finished = true; }
+  if (stream->error == EMPTY) { finished = true; }
   // If we had any other kind of error (not EMPTY) then we want
   // to pass it along to the operator* and we cannot mark the result
   // as "finished" just yet.
@@ -180,11 +184,10 @@ inline void document_stream::start() noexcept {
   if (error) { return; }
   error = parser->ensure_capacity(batch_size);
   if (error) { return; }
-
   // Always run the first stage 1 parse immediately
   batch_start = 0;
   error = run_stage1(*parser, batch_start);
-  if(error == EMPTY) {
+  while(error == EMPTY) {
     // In exceptional cases, we may start with an empty block
     batch_start = next_batch_start();
     if (batch_start >= len) { return; }
@@ -201,22 +204,28 @@ inline void document_stream::start() noexcept {
     if (error) { return; }
   }
 #endif // SIMDJSON_THREADS_ENABLED
-
   next();
 }
 
 simdjson_really_inline size_t document_stream::iterator::current_index() const noexcept {
-  return stream.doc_index;
+  return stream->doc_index;
 }
 
 simdjson_really_inline std::string_view document_stream::iterator::source() const noexcept {
-  size_t next_doc_index = stream.batch_start + stream.parser->implementation->structural_indexes[stream.parser->implementation->next_structural_index];
-  return std::string_view(reinterpret_cast<const char*>(stream.buf) + current_index(), next_doc_index - current_index() - 1);
+  const char* start = reinterpret_cast<const char*>(stream->buf) + current_index();
+  bool object_or_array = ((*start == '[') || (*start == '{'));
+  if(object_or_array) {
+    size_t next_doc_index = stream->batch_start + stream->parser->implementation->structural_indexes[stream->parser->implementation->next_structural_index - 1];
+    return std::string_view(start, next_doc_index - current_index() + 1);
+  } else {
+    size_t next_doc_index = stream->batch_start + stream->parser->implementation->structural_indexes[stream->parser->implementation->next_structural_index];
+    return std::string_view(reinterpret_cast<const char*>(stream->buf) + current_index(), next_doc_index - current_index() - 1);
+  }
 }
 
 
 inline void document_stream::next() noexcept {
-  // We always enter at once once in an error condition.
+  // We always exit at once, once in an error condition.
   if (error) { return; }
 
   // Load the next document from the batch
@@ -242,18 +251,25 @@ inline void document_stream::next() noexcept {
     error = parser->implementation->stage2_next(parser->doc);
   }
 }
+inline size_t document_stream::size_in_bytes() const noexcept {
+  return len;
+}
+
+inline size_t document_stream::truncated_bytes() const noexcept {
+  if(error == CAPACITY) { return len - batch_start; }
+  return parser->implementation->structural_indexes[parser->implementation->n_structural_indexes] - parser->implementation->structural_indexes[parser->implementation->n_structural_indexes + 1];
+}
 
 inline size_t document_stream::next_batch_start() const noexcept {
   return batch_start + parser->implementation->structural_indexes[parser->implementation->n_structural_indexes];
 }
 
 inline error_code document_stream::run_stage1(dom::parser &p, size_t _batch_start) noexcept {
-  // If this is the final batch, pass partial = false
   size_t remaining = len - _batch_start;
   if (remaining <= batch_size) {
-    return p.implementation->stage1(&buf[_batch_start], remaining, false);
+    return p.implementation->stage1(&buf[_batch_start], remaining, stage1_mode::streaming_final);
   } else {
-    return p.implementation->stage1(&buf[_batch_start], batch_size, true);
+    return p.implementation->stage1(&buf[_batch_start], batch_size, stage1_mode::streaming_partial);
   }
 }
 
