@@ -1,2 +1,2 @@
-SIMDJSON_UNTARGET_REGION
+SIMDJSON_UNTARGET_HASWELL
 #undef SIMDJSON_IMPLEMENTATION
