@@ -7,12 +7,12 @@
 namespace simdjson {
 
 #if SIMDJSON_EXCEPTIONS
-
+#ifndef SIMDJSON_DISABLE_DEPRECATED_API
 [[deprecated("Use padded_string::load() instead")]]
 inline padded_string get_corpus(const char *path) {
   return padded_string::load(path);
 }
-
+#endif // SIMDJSON_DISABLE_DEPRECATED_API
 #endif // SIMDJSON_EXCEPTIONS
 
 } // namespace simdjson
